<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format">

<xsl:variable name="font" select="Cyrillic"/>

<!-- display title in different formats -->
<xsl:template match="Title">
<xsl:choose>
<!-- manual title -->
<xsl:when test="parent::Manual">
  <fo:block font-size="24.0pt" line-height="24pt" space-after.optimum="25pt" text-align="right" padding-top="3pt"><xsl:apply-templates/></fo:block>
</xsl:when>

<!-- chapter title -->
<xsl:when test="parent::Chapter">
<fo:block font-size="24.0pt" line-height="24pt" text-align="right" padding-top="3pt" id="Chapter_{../@id}">
<xsl:if test="not(starts-with(@id, 'appendix')) and not(starts-with(@id, 'biblio'))"><xsl:number count="Chapter" format="1"/></xsl:if>
</fo:block>
<fo:block font-size="24.0pt" line-height="24pt" space-before.optimum="25pt" text-align="right" padding-top="3pt" text-indent="3cm"><xsl:apply-templates/></fo:block>
</xsl:when>

<!-- sections' title, with section number -->
<xsl:when test="parent::Section and not(../Manual)">
<fo:block font-size="12.0pt" text-align="left" padding-top="3pt" space-before.optimum="25pt" id="Section_{../@id}">
<xsl:if test="not(starts-with(../../../@id, 'appendix'))"><xsl:number level="multiple" count="Chapter|Chapter/Content/Section" format="1.1"/>&#160;&#160;</xsl:if>
<xsl:apply-templates select="Text"/>
</fo:block>
</xsl:when>

<!-- tables(not hidden) title, in grey background and with unique number -->
<xsl:when test="parent::Table">
<fo:block id="Table_{../@id}" font-size="10.0pt" font-weight="bold" line-height="15pt" background-color="grey" margin-left="25pt" space-before="10pt" space-after="3pt">
Table&#160;<xsl:number level="any" count="Table[not(@class='hidden')]"/>&#160;&#160;<xsl:apply-templates select="Text"/><xsl:value-of select="SubTitle"/>
</fo:block>
</xsl:when>

<!-- title of subsections -->
<xsl:when test="parent::Subsection">
<fo:block font-size="10pt" font-weight="bold" text-align="left" id="{../@id}" space-before.optimum="25pt">
<xsl:apply-templates select="Text"/>
</fo:block>
</xsl:when>

<!-- title of parts -->
<xsl:when test="parent::Part">
<fo:block font-size="10pt" font-weight="bold" text-align="left" id="{../@id}" space-before.optimum="15pt">
<xsl:apply-templates select="Text"/>
</fo:block>
</xsl:when>	
</xsl:choose>
</xsl:template>



<xsl:template match="Title" mode="Figure">
<xsl:param name="no"/>
<fo:block id="Figure_{../@id}" font-size="10.0pt" font-weight="bold" line-height="15pt" background-color="grey" margin-left="25pt" space-before="10pt" space-after="3pt">
Figure&#160;<xsl:value-of select="$no"/><xsl:if test="SubTitle"><xsl:value-of select="SubTitle"/></xsl:if>&#160;<xsl:apply-templates select="Text"/>
</fo:block>
</xsl:template>



<!-- deal with various text paragraph formats -->
<xsl:template match="Text">
<xsl:choose>
<!-- these text display directly -->
<xsl:when test="parent::Title|parent::Author|parent::Affiliation|parent::Version|parent::Date|parent::Bullet|parent::Footnote"><xsl:apply-templates/></xsl:when>

<!-- text in table cells display with 11pt font size -->
<xsl:when test="parent::th">
<fo:block font-size="11.0pt"><xsl:apply-templates select="node()"/></fo:block>
</xsl:when>

<!-- indented text -->
<xsl:when test="parent::Indented">
<fo:block font-size="11.0pt" color="black" text-align="justify" space-before.optimum="15pt" margin-left="35pt">
<xsl:apply-templates/>
</fo:block>
</xsl:when>

<!-- normal text paragraphs -->
<xsl:otherwise>
<fo:block font-size="11.0pt" color="black" text-align="justify" space-before.optimum="15pt" margin-left="25pt">
<xsl:apply-templates/>
</fo:block>
</xsl:otherwise>
</xsl:choose>
</xsl:template>



<xsl:template match="Section|Content"><xsl:apply-templates/></xsl:template>


<xsl:template match="Version">
<fo:block font-size="12.0pt" font-weight="bold" color="gray" space-before.optimum="15pt" padding-top="3pt" margin-left="25pt" text-align-last="justify">
<xsl:apply-templates/><fo:leader leader-pattern="space"/><xsl:value-of select="following-sibling::Date"/>
</fo:block>
</xsl:template>


<xsl:template match="BulletList">
<xsl:if test="not(parent::Bullet)">
<fo:list-block start-indent="2cm" provisional-distance-between-starts="12pt" font-family="{$font}" space-before.optimum="15pt">
<xsl:for-each select="Bullet">
<fo:list-item><fo:list-item-label end-indent="label-end()">
<fo:block><fo:inline font-family="Symbol">&#x2022;</fo:inline></fo:block>
</fo:list-item-label>
<fo:list-item-body start-indent="body-start()">				
<fo:block><xsl:apply-templates/></fo:block>

<!-- if contains a child BulletList node -->
<xsl:if test="BulletList">					
<xsl:for-each select="BulletList">
<fo:list-block start-indent="3cm" provisional-distance-between-starts="12pt" font-family="{$font}" space-before.optimum="5pt">
<xsl:for-each select="Bullet">
<fo:list-item><fo:list-item-label end-indent="label-end()">
<fo:block><fo:inline font-family="Symbol">&#x2022;</fo:inline></fo:block>
</fo:list-item-label>
<fo:list-item-body start-indent="body-start()">
<fo:block><xsl:apply-templates/></fo:block>
</fo:list-item-body>
</fo:list-item>
</xsl:for-each>
</fo:list-block>	
</xsl:for-each>
</xsl:if>

</fo:list-item-body>
</fo:list-item>
</xsl:for-each>
</fo:list-block>
</xsl:if>
</xsl:template>



<xsl:template match="NumberedList">
<fo:list-block start-indent="2cm" provisional-distance-between-starts="12pt" font-family="{$font}" space-before.optimum="15pt">
<xsl:for-each select="NumberedItem">
<fo:list-item><fo:list-item-label end-indent="label-end()">
<fo:block><fo:inline font-family="Symbol"><xsl:number/>.</fo:inline></fo:block>
</fo:list-item-label>
<fo:list-item-body start-indent="body-start()">
<xsl:choose>
<!-- does not contain child (indented) Text or CodeLines -->
<xsl:when test="(count(Text)+count(CodeLine))=1">
<fo:block><xsl:apply-templates select="Text/node()"/></fo:block>
</xsl:when>

<!-- contains child (indented) Text or CodeLines -->
<xsl:otherwise>
<fo:block><xsl:apply-templates select="Text[1]/node()"/></fo:block>
<fo:list-block start-indent="3cm" provisional-distance-between-starts="12pt" font-family="{$font}" space-before.optimum="15pt" space-after.optimum="15pt">
<xsl:for-each select="child::*[position()>1]">
<fo:list-item><fo:list-item-label end-indent="label-end()"><fo:block/></fo:list-item-label>
<fo:list-item-body><fo:block><xsl:apply-templates/></fo:block></fo:list-item-body>
</fo:list-item>
</xsl:for-each>
</fo:list-block>
</xsl:otherwise>
</xsl:choose>
</fo:list-item-body>
</fo:list-item>
</xsl:for-each>
</fo:list-block>
</xsl:template>



<xsl:template match="b"><fo:inline font-weight="bold"><xsl:apply-templates/></fo:inline></xsl:template>

<xsl:template match="i"><fo:inline font-style="italic"><xsl:apply-templates/></fo:inline></xsl:template>

<xsl:template match="u"><fo:inline text-decoration="underline" color="blue"><xsl:apply-templates/></fo:inline></xsl:template>

<xsl:template match="CodeLine">
<fo:block font-size="8.0pt" font-family="Courier" color="black" text-align="left">
<xsl:choose>
<xsl:when test="parent::NumberedItem or parent::Bullet"/>
<!-- normal code lines -->
<xsl:when test="not(parent::Figure) and not(parent::th)">
<xsl:attribute name="margin-left">50pt</xsl:attribute>
<xsl:if test="not(name(preceding-sibling::*[position()=1])='CodeLine')">
<xsl:attribute name="space-before.optimum">8pt</xsl:attribute>
</xsl:if>
</xsl:when>

<!-- codeline in a hidden table -->
<xsl:when test="parent::th and ../../../../@class='hidden'">
<xsl:attribute name="margin-left">10pt</xsl:attribute>
<xsl:attribute name="space-before.optimum">8pt</xsl:attribute>
<xsl:attribute name="space-after.optimum">8pt</xsl:attribute>
</xsl:when>
<!-- normal table -->
<xsl:otherwise>
<xsl:if test="not(parent::th)">
<xsl:attribute name="space-before.optimum">3pt</xsl:attribute>
</xsl:if>
</xsl:otherwise>
</xsl:choose>

<xsl:apply-templates/>
</fo:block>

<!--
<xsl:param name="withLineNumber">n</xsl:param>
<xsl:choose>
<xsl:when test="not(parent::Figure) and not(parent::th)">
<fo:block font-size="8.0pt" font-family="Courier" color="black" text-align="left" margin-left="50pt">
<xsl:if test="not(name(preceding-sibling::*[position()=1])='CodeLine')">
<xsl:attribute name="space-before.optimum">8pt</xsl:attribute>
</xsl:if>
<xsl:apply-templates/>
</fo:block>
</xsl:when>

<xsl:when test="parent::th and ../../../../@class='hidden'">
<fo:block font-size="8.0pt" font-family="Courier" color="black" 
text-align="left" margin-left="10pt" space-before.optimum="8pt" space-after.optimum="8pt">
<xsl:apply-templates/>
</fo:block>
</xsl:when>

<xsl:otherwise>
<fo:block font-size="8.0pt" font-family="Courier" color="black" text-align="left">
<xsl:if test="not(parent::th)">
<xsl:attribute name="space-before.optimum">3pt</xsl:attribute>
</xsl:if>
<xsl:if test="$withLineNumber='y'"><xsl:number/></xsl:if>
<xsl:apply-templates/>
</fo:block>
</xsl:otherwise>
</xsl:choose>
-->
</xsl:template>





<!-- table -->
<xsl:template match="Table">
<fo:table table-layout="fixed" width="100%">
<fo:table-column column-width="proportional-column-width(1)"/>
<fo:table-body>

<!-- table title -->

<fo:table-row>
<!-- always keep the table title with the table -->
<xsl:if test="not(normalize-space(Title/Text)='')"><xsl:attribute name="keep-with-next">always</xsl:attribute></xsl:if>				
<fo:table-cell>
<fo:block>
<xsl:if test="not(normalize-space(Title/Text)='')"><xsl:apply-templates select="Title"/></xsl:if>
</fo:block>
</fo:table-cell>
</fo:table-row>

<!-- table content -->

<fo:table-row>
<fo:table-cell>
<fo:block>
<!-- use nested table to center-align the table -->
<fo:table table-layout="fixed" width="100%">
<fo:table-column column-width="25pt"/>
<fo:table-column column-width="60pt"/>				
<fo:table-body>
<fo:table-row>
<fo:table-cell column-number="1"/>
<fo:table-cell column-number="2">
<fo:table table-layout="fixed">
<xsl:for-each select="TableContent/tr[1]/th">
<fo:table-column>

<!-- calculate the column width based on the width in the original html file -->

<xsl:attribute name="column-width">
<xsl:value-of select="concat(number(@width) * 0.8, 'px')"/>
</xsl:attribute>
</fo:table-column>
</xsl:for-each>
<fo:table-body>
<xsl:for-each select="TableContent/tr">

<!-- this is to ensure no breaks in the table -->

<fo:table-row keep-with-next="always">
<xsl:for-each select="th">
<xsl:choose>

<!-- when the cell contains only text -->

<xsl:when test="not(../../../@class='hidden') and not(File)">
<fo:table-cell column-number="{position()}" vertical-align="top" padding="2pt" border="0.5pt" border-color="black" border-style="solid">
<xsl:if test="position()=1 and normalize-space(../../tr[1]/th[1])=''"><xsl:attribute name="text-align">right</xsl:attribute></xsl:if>
<xsl:if test="position()>1 and not(last())"><xsl:attribute name="text-align">left</xsl:attribute></xsl:if>
<xsl:if test="last()"><xsl:attribute name="text-align">justify</xsl:attribute></xsl:if>
<xsl:apply-templates/>
</fo:table-cell>
</xsl:when>

<!-- when the cell is an image --> 

<xsl:when test="not(../../../@class='hidden') and File">
<fo:table-cell column-number="{position()}" vertical-align="top" text-align="right" padding="2pt" border="0.5pt" border-color="black" border-style="solid">
<xsl:apply-templates/>
</fo:table-cell>
</xsl:when>

<!-- deal with hidden table -->

<xsl:when test="../../../@class='hidden'">
<fo:table-cell column-number="{position()}" vertical-align="top" padding="2pt">
<xsl:if test="position()=1"><xsl:attribute name="text-align">left</xsl:attribute></xsl:if>
<xsl:if test="position()>1"><xsl:attribute name="text-align">justify</xsl:attribute></xsl:if>
<xsl:apply-templates/>
</fo:table-cell></xsl:when></xsl:choose></xsl:for-each></fo:table-row></xsl:for-each></fo:table-body></fo:table></fo:table-cell>
</fo:table-row></fo:table-body></fo:table></fo:block></fo:table-cell></fo:table-row></fo:table-body></fo:table>
</xsl:template>



<!-- Figure -->
<xsl:template match="Figure">
<fo:table table-layout="fixed" width="100%">
<fo:table-column column-width="proportional-column-width(1)"/>
<fo:table-body>

<!-- figure title -->
<!-- number of current figure = total_number_of_previous_figure - total_number_of_previous_sub_figures-->
<xsl:variable name="temp_total_figs"><xsl:number count="Figure" level="any"/></xsl:variable>       
<xsl:variable name="temp_num_of_sub_figs">
  <xsl:number count="Figure[Title/SubTitle and substring(translate(@id, '_0123456789', '99999999999'), string-length(@id)-1, string-length(@id))='99']" level="any"/>
</xsl:variable>  	  	
<xsl:variable name="no"><xsl:value-of select="$temp_total_figs - $temp_num_of_sub_figs"/></xsl:variable>

<fo:table-row keep-with-next="always"><fo:table-cell><fo:block>
<xsl:apply-templates select="Title" mode="Figure"><xsl:with-param name="no" select="$no"/></xsl:apply-templates>
</fo:block></fo:table-cell></fo:table-row>


<fo:table-row><fo:table-cell>
<xsl:if test="File"><xsl:apply-templates select="File"/></xsl:if>
<xsl:if test="CodeLine">
<fo:block text-align="left">
<fo:table start-indent="3pt" table-layout="fixed" width="100%" border-spacing="3pt">
<fo:table-column column-width="proportional-column-width(1)"/>
<fo:table-column column-width="100mm"/>
<fo:table-column column-width="proportional-column-width(1)"/>
<fo:table-body>
<fo:table-row>
<fo:table-cell column-number="2">

<xsl:if test="not(@calss='withLineNumber')">
<xsl:attribute name="border-width">0.1pt</xsl:attribute><xsl:attribute name="border-style">solid</xsl:attribute>
<xsl:attribute name="border-color">black</xsl:attribute><xsl:attribute name="border-collapse">separate</xsl:attribute>
</xsl:if>

<xsl:choose>
<!-- display the codeline number besides each codeline using nested table -->
<xsl:when test="@class='withLineNumber'">
<fo:table start-indent="3pt" table-layout="fixed" width="100%" border-spacing="3pt">
<fo:table-column column-width="28pt"/>											  
<fo:table-column column-width="proportional-column-width(1)" border-width="0.1pt" border-style="solid" border-color="black" border-collapse="separate"/>
<fo:table-body>
<xsl:for-each select="CodeLine">
<fo:table-row>
<fo:table-cell column-number="1">
<fo:block font-size="8pt" font-family="Courier" space-before.optimum="3pt" text-align="right"><xsl:number/></fo:block>
</fo:table-cell>
<fo:table-cell column-number="2"><xsl:apply-templates select="."/></fo:table-cell>
</fo:table-row>
</xsl:for-each>
</fo:table-body>
</fo:table>										
</xsl:when>

<xsl:otherwise><xsl:apply-templates select="CodeLine"/></xsl:otherwise>
</xsl:choose></fo:table-cell></fo:table-row></fo:table-body></fo:table></fo:block></xsl:if>
</fo:table-cell></fo:table-row></fo:table-body></fo:table>
</xsl:template>




<xsl:template match="File">
<xsl:variable name="height"><xsl:value-of select="@height"/>em</xsl:variable>
<xsl:variable name="width"><xsl:value-of select="@width"/>em</xsl:variable>
<fo:block display-align="center" text-align="center" space-after.optimum="15pt" space-before.optimum="3pt">
<fo:external-graphic display-align="center" src="{@url}" content-height="{$height}" content-width="{$width}"/>
</fo:block>
</xsl:template>



<!-- reference in the text to a footnote -->
<xsl:template match="FootnoteRef">
<xsl:variable name="no"><xsl:value-of select="@id"/></xsl:variable>
<fo:footnote>
<fo:inline baseline-shift="super" font-size="smaller">(<xsl:value-of select="$no"/>)</fo:inline>
<fo:footnote-body>
<fo:list-block provisional-label-separation="0pt" provisional-distance-between-starts="18pt" space-after.optimum="6pt">
<fo:list-item>
<fo:list-item-label end-indent="label-end()">
<fo:block>(<xsl:number count="//FootnoteRef" level="any"/>)</fo:block>
</fo:list-item-label>
<fo:list-item-body start-indent="body-start()">
<fo:block><xsl:apply-templates select="//Footnote[@id=$no]"/></fo:block>
</fo:list-item-body>
</fo:list-item>
</fo:list-block>
</fo:footnote-body>
</fo:footnote>
</xsl:template>


<xsl:template match="Footnote"><xsl:apply-templates/></xsl:template>


<xsl:template match="BibItem">
<fo:block font-size="11.0pt" color="black" text-align="justify" space-before.optimum="15pt" margin-left="25pt"><xsl:apply-templates/></fo:block>
</xsl:template>


<xsl:template match="Author">
<fo:block font-size="15pt" color="black" text-align="right" space-before.optimum="25pt"><xsl:apply-templates/></fo:block>
</xsl:template>


<xsl:template match="Affiliation">
<fo:block font-size="10.0pt" color="black" text-align="right" space-after.optimum="125pt" space-before.optimum="25pt"><xsl:apply-templates/></fo:block>
</xsl:template>


<xsl:template match="Text" mode="heading"><fo:block font-size="18.0pt" color="gray" text-align="right"><xsl:apply-templates/></fo:block></xsl:template>


<xsl:template match="text()"><xsl:value-of select="."/></xsl:template>

<xsl:template match="br"><fo:block/></xsl:template>
</xsl:stylesheet>
